from Acquisition import aq_inner
from Acquisition import aq_parent
from DateTime import DateTime
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.i18nl10n import monthname_msgid
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from ftw.events import _
from ftw.events.interfaces import IEventListingView
from ftw.events.interfaces import IEventPage
from plone.app.portlets.interfaces import IPortletPermissionChecker
from plone.app.portlets.portlets import base
from plone.directives.form.form import SchemaAddForm
from plone.directives.form.form import SchemaEditForm
from plone.memoize.view import memoize
from plone.portlets.interfaces import IPortletDataProvider
from z3c.form import button
from z3c.form import form as z3cform
from zope import schema
from zope.component import getMultiAdapter
from zope.i18n import translate
from zope.interface import implements


def zLocalizedTime(request, time, long_format=False):
    """Convert time to localized time
    """
    month_msgid = monthname_msgid(time.strftime('%m'))
    month = translate(month_msgid, domain='plonelocales',
                      context=request)

    return month.encode('utf-8')


class ArchiveSummary(object):

    def __init__(self, context, request, interfaces, date_field, view):
        self.context = context
        self.request = request
        self.interfaces = interfaces
        self.date_field = date_field
        self.view = view
        self.selected_year = None
        self.selected_month = None

    def __call__(self):
        self._set_selected_archive()

        entries = self._get_archive_entries()
        counter = self._count_entries(entries)

        result = []
        year_numbers = sorted(counter, reverse=True)

        for year_number in year_numbers:

            year = counter.get(year_number)
            months = year.get('months')
            month_numbers = sorted(months, reverse=True)
            month_list = []

            for month_number in month_numbers:
                date = '%s/%s/01' % (year_number, month_number)

                month_list.append(dict(
                    title=zLocalizedTime(self.request, DateTime(date)),
                    number=months.get(month_number),
                    url=self._get_archive_url(date),
                    mark=[self.selected_year, self.selected_month] == [
                        year_number, month_number]
                ))

            result.append(dict(
                title=year_number,
                number=year.get('num'),
                months=month_list,
                mark=self.selected_year == year_number
            ))

        return result

    def _set_selected_archive(self):
        selected_archive = self.request.get('archive')
        if selected_archive:
            self.selected_year = selected_archive.split('/')[0]
            self.selected_month = selected_archive.split('/')[1]

    def _get_archive_url(self, date):
        url = '{url}/{view}?archive={date}'.format(
            url=self.context.absolute_url(),
            view=self.view.__name__,
            date=date)

        portlet = self.request.get('portlet', None)
        manager = self.request.get('manager', None)
        if portlet:
            url += '&portlet={0}'.format(portlet)
        if manager:
            url += '&manager={0}'.format(manager)

        return url

    def _get_archive_entries(self):
        catalog = getToolByName(self.context, 'portal_catalog')
        query = self._get_query()
        if 'start' in query:
            del query['start']
        if 'end' in query:
            del query['end']
        return catalog(**query)

    def _get_query(self):
        return self.view.get_query()

    def _count_entries(self, entries):
        """Return a summary map like:

        {'2009': {
            'num': 6,
            'months': {
                '01': 4,
                '02': 2}
            }
        }
        """
        summary = {}

        for entry in entries:

            date = getattr(entry, self.date_field)
            if not date:
                continue

            year_name = date.strftime('%Y')
            month_name = date.strftime('%m')

            year_summary = summary.get(year_name, {})
            month_summary = year_summary.get('months', {})

            # Increase month
            month_summary.update(
                {month_name: month_summary.get(month_name, 0) + 1})
            year_summary.update({'months': month_summary})

            # Increase year
            year_summary.update({'num': year_summary.get('num', 0) + 1})
            summary.update({year_name: year_summary})

        return summary


class IEventsArchivePortlet(IPortletDataProvider):
    """Archive portlet interface.
    """

    portlet_title = schema.TextLine(
        title=_(u'title', default=u'Title'),
        required=True,
        default=u'',
    )


class Assignment(base.Assignment):
    implements(IEventsArchivePortlet)

    def __init__(self, portlet_title):
        self._portlet_title = portlet_title

    @property
    def portlet_title(self):
        return (self._portlet_title if hasattr(self, '_portlet_title') else
                _(u'archive', default=u'Archive'))

    @portlet_title.setter
    def portlet_title(self, value):
        self._portlet_title = value

    @property
    def title(self):
        return u'Event Archive Portlet ({0})'.format(self.portlet_title)


class Renderer(base.Renderer):
    def __init__(self, context, request, view, manager, data):
        self.context = context
        self.data = data
        self.request = request
        self.view = view

    @property
    def available(self):
        """Show the portlet only if we're on a news listing view and
        there is some content to be displayed.
        """
        is_event_listing_view = IEventListingView.providedBy(self.view)
        return is_event_listing_view and bool(self.get_items())

    @memoize
    def get_items(self):
        """Returns an ordered list of summary info per month."""
        summary = ArchiveSummary(
            context=self.context,
            request=self.request,
            interfaces=[IEventPage.__identifier__],
            date_field='start',
            view=self.view)()

        items = [{
            'title': year['title'],
            'count': year['number'],
            'class': 'event-year expanded' if year['mark'] else 'event-year',
            'months_expanded': 'event-months expanded' if year['mark'] else 'event-months',
            'months': [{
                'title': month['title'],
                'url': month['url'],
                'class': 'event-month highlight' if month['mark'] else 'event-month',
                'month': month['title'],
                'count': month['number'],
            } for month in year['months']],
        } for year in summary]
        return items

    render = ViewPageTemplateFile('templates/events_archive_portlet.pt')


class AddForm(SchemaAddForm):
    label = _(u'event_archive_portlet_add', default=u'Add Event Archive Portlet')
    schema = IEventsArchivePortlet

    def __init__(self, context, request):
        super(AddForm, self).__init__(context, request)
        self.status = None
        self._finishedAdd = None

    def __call__(self):
        IPortletPermissionChecker(aq_parent(aq_inner(self.context)))()
        return super(AddForm, self).__call__()

    def nextURL(self):
        editview = aq_parent(aq_inner(self.context))
        context = aq_parent(aq_inner(editview))
        url = str(getMultiAdapter((context, self.request),
                                  name=u'absolute_url'))
        return url + '/@@manage-portlets'

    def add(self, object_):
        ob = self.context.add(object_)
        self._finishedAdd = True
        return ob

    def create(self, data):
        return Assignment(
            portlet_title=data.get('portlet_title'),
        )


class EditForm(SchemaEditForm):
    label = _(u'event_archive_portlet_edit', default=u'Edit Event Archive Portlet')
    schema = IEventsArchivePortlet

    def __init__(self, context, request):
        super(EditForm, self).__init__(context, request)
        self.status = None
        self._finishedAdd = None

    def __call__(self):
        IPortletPermissionChecker(aq_parent(aq_inner(self.context)))()
        return super(EditForm, self).__call__()

    def nextURL(self):
        editview = aq_parent(aq_inner(self.context))
        context = aq_parent(aq_inner(editview))
        url = str(getMultiAdapter((context, self.request),
                                  name=u'absolute_url'))
        return url + '/@@manage-portlets'

    @button.buttonAndHandler(_(u'events_portlet_edit_form_save_label',
                               default=u'Save'),
                             name='apply')
    def handleSave(self, action):
        data, errors = self.extractData()
        if errors:
            self.status = self.formErrorsMessage
            return
        changes = self.applyChanges(data)
        if changes:
            self.status = 'Changes saved'
        else:
            self.status = 'No changes'

        nextURL = self.nextURL()
        return self.request.response.redirect(nextURL)

    @button.buttonAndHandler(_(u'events_portlet_edit_form_cancel_label',
                               default=u'Cancel'),
                             name='cancel_add')
    def handleCancel(self, action):
        nextURL = self.nextURL()
        return self.request.response.redirect(nextURL)

    updateActions = z3cform.EditForm.updateActions



from ftw.upgrade import UpgradeStep


class UseFtwKeywordwidgetOnTheEventListingBlock(UpgradeStep):
    """Use ftw.keywordwidget on the event listing block.
    """

    def __call__(self):
        self.setup_install_profile('profile-ftw.keywordwidget:default')
        self.install_upgrade_profile()



from Acquisition import aq_inner, aq_parent
from ftw.events import _
from ftw.events.interfaces import IEventListingBlock
from ftw.keywordwidget.widget import KeywordFieldWidget
from ftw.referencewidget.sources import DefaultSelectable
from ftw.referencewidget.sources import ReferenceObjSourceBinder
from ftw.referencewidget.widget import ReferenceWidgetFactory
from ftw.simplelayout.contenttypes.behaviors import IHiddenBlock
from plone import api
from plone.autoform import directives
from plone.autoform.interfaces import IFormFieldProvider
from plone.dexterity.content import Item
from plone.directives import form
from plone.uuid.interfaces import IUUID
from Products.CMFPlone.interfaces.syndication import IFeedSettings
from Products.CMFPlone.interfaces.syndication import ISyndicatable
from z3c.relationfield import RelationChoice
from zope import schema
from zope.interface import alsoProvides
from zope.interface import implements
from zope.interface import invariant, Invalid


class FilterByPathSelectable(DefaultSelectable):

    def is_selectable(self):
        """ Allow to reference any path"""
        return True


class IEventListingBlockSchema(form.Schema):
    form.fieldset(
        'filter',
        label=_(u'event_listing_fieldset_filter_label',
                default=u'Filter'),
        fields=[
            'filter_by_path',
            'current_context',
            'subjects',
        ]
    )

    form.fieldset(
        'link',
        label=_(u'event_listing_fieldset_link_label',
                default=u'Link'),
        fields=[
            'show_more_items_link',
            'link_to_more_items',
            'more_items_link_label',
            'more_items_view_title',
        ]
    )

    form.fieldset(
        'advanced',
        label=_(u'event_listing_fieldset_advanced_label',
                default=u'Advanced'),
        fields=[
            'exclude_past_events',
            'hide_empty_block',
        ]
    )

    title = schema.TextLine(
        title=_(u'event_listing_config_title_label', default=u'Title'),
        description=u'',
        required=True,
        default=u'',
    )

    show_title = schema.Bool(
        title=_(u'event_listing_block_show_title_label', default=u'Show title'),
        default=True,
        required=False,
    )

    exclude_past_events = schema.Bool(
        title=_(u'label_exclude_past_events', default=u'Exclude past events'),
        default=True,
        required=False,
    )

    form.widget(filter_by_path=ReferenceWidgetFactory)
    filter_by_path = schema.List(
        title=_(u'event_listing_config_filter_path_label',
                default=u'Limit to path'),
        description=_(u'event_listing_config_filter_path_description',
                      default=u'Only show content from a specific path.'),
        value_type=RelationChoice(
            source=ReferenceObjSourceBinder(
                selectable_class=FilterByPathSelectable),
        ),
        required=False,
        default=[],
        missing_value=[],
    )

    current_context = schema.Bool(
        title=_(u'event_listing_config_filter_current_context_label',
                default=u'Limit to current context'),
        description=_(
            u'event_listing_config_filter_current_context_description',
            default=u'Only show items from the current context.'),
        default=True,
    )

    quantity = schema.Int(
        title=_(u'event_listing_config_quantity_label', default=u'Quantity'),
        description=_(u'event_listing_config_quantity_description',
                      default=u'The number of items to be shown at most. '
                              u'Enter 0 for no limitation.'),
        default=5,
    )

    directives.widget('subjects', KeywordFieldWidget)
    subjects = schema.List(
        title=_(u'event_listing_config_subjects_label',
                default=u'Filter by subject'),
        description=_(u'event_listing_config_subjects_description',
                      default=u'Only items with the selected subjects will '
                              u'be shown.'),
        value_type=schema.Choice(vocabulary='ftw.events.vocabulary.subjects'),
        required=False,
        default=[],
        missing_value=[],
    )

    show_description = schema.Bool(
        title=_(u'event_listing_config_show_description_label',
                default=u'Show the description of the item'),
        default=True,
    )

    description_length = schema.Int(
        title=_(u'event_listing_config_description_length_label',
                default=u'Length of the description'),
        description=_(u'event_listing_config_description_length_description',
                      default=u'The maximum length of the item\'s '
                              u'description. Longer descriptions will be '
                              u'cropped. Enter 0 for no limitation.'),
        default=50,
    )

    show_more_items_link = schema.Bool(
        title=_(u'event_listing_config_show_more_items_link',
                default=u'Show link to more items'),
        description=_(u'event_listing_show_more_items_link_description',
                      default=u'Render a link to a page which renders more '
                              u'items (only if there is at least one item).'),
        default=False,
    )

    form.widget(link_to_more_items=ReferenceWidgetFactory)
    link_to_more_items = RelationChoice(
        title=_(u'event_listing_config_items_link', default=u'Link to more items'),
        source=ReferenceObjSourceBinder(
            selectable_class=FilterByPathSelectable),
        default=None,
        required=False,
    )

    more_items_link_label = schema.TextLine(
        title=_(u'label_more_items_link_label',
                default=u'Label for the "more items" link'),
        description=_(u'description_more_items_link_label',
                      default=u'This custom label will not be translated.'),
        required=False,
    )

    more_items_view_title = schema.TextLine(
        title=_(u'label_more_items_view_title',
                default=u'Title of the view behind the "more items" link'),
        description=_(u'description_more_items_view_title',
                      default=u'This title will not be translated.'),
        required=False,
    )

    show_rss_link = schema.Bool(
        title=_(u'label_show_rss_link',
                default=u'Link to RSS feed'),
        description=_(u'description_show_rss_link',
                      default=u'Render a link to the RSS feed of the events.'),
        default=False,
        required=False,
    )

    hide_empty_block = schema.Bool(
        title=_(u'label_hide_empty_block',
                default=u'Hide empty block'),
        description=_(u'description_hide_empty_block',
                      default=u'Hide the block if there are not events to be shown.'),
        default=False,
        required=False,
    )

    @invariant
    def is_either_path_or_context(obj):
        """Checks if not both path and current context are defined.
        """
        if obj.current_context and obj.filter_by_path:
            raise Invalid(_(
                u'event_listing_config_current_context_and_path_error',
                default=u'You cannot filter by path and current context '
                        u'at the same time.')
            )

alsoProvides(IEventListingBlockSchema, IFormFieldProvider)


class EventListingBlock(Item):
    implements(IEventListingBlock, ISyndicatable, IHiddenBlock)

    @property
    def is_hidden(self):
        if not self.hide_empty_block:
            return False

        # For unknown reason, `self` is not acquisition wrapped. We
        # need to get an acquisition wrapped event listing block we
        # work with.
        obj = api.content.get(UID=IUUID(self))

        if self.user_can_edit_block(obj):
            # Editors must always see the block or they cannot edit it anymore.
            return True

        items = api.content.get_view(
            name='block_view', context=obj, request=obj.REQUEST
        ).get_items()
        return self.hide_empty_block and not items

    @is_hidden.setter
    def is_hidden(self, value):
        """
        This is a dummy setter method in case somebody activates the IHiddenBlock
        behavior from ftw.simplelayout on the event listing block which makes no
        sense. It should not be a use case to allow hiding event listing blocks
        because the block does not really contain the objects, it only displays
        them from a different location. So if an event listing block ist not desired
        it should be removed from the content page instead.
        """
        raise Exception(
            'You are not allowed to add the IHiddenBlock behavior on the event listing block.'
        )

    def user_can_edit_block(self, obj):
        container = aq_parent(aq_inner(obj))
        return api.user.has_permission(
            'Modify portal content',
            obj=container,
        )


def enable_syndication(event_listing_block, event=None):
    """
    Enables syndication on the given event listing block. This
    must be called from a subscriber when an event listing block is
    created so the syndication is enabled by default on the event
    listing blocks.
    """
    settings = IFeedSettings(event_listing_block)
    settings.enabled = True
    settings.feed_types = ('RSS', 'rss.xml', 'atom.xml')



from datetime import datetime
from datetime import timedelta
from ftw.builder import builder_registry
from ftw.builder.dexterity import DexterityBuilder
from ftw.simplelayout.tests import builders
from ftw.testing import IS_PLONE_5


class EventFolderBuilder(DexterityBuilder):
    portal_type = 'ftw.events.EventFolder'

builder_registry.register('event folder', EventFolderBuilder)


class EventPageBuilder(DexterityBuilder):
    portal_type = 'ftw.events.EventPage'

    def __init__(self, *args, **kwargs):
        super(EventPageBuilder, self).__init__(*args, **kwargs)
        # ftw.builders auto-discovery does not work for recurrence:
        self.having(recurrence='')
        # using greenwich, since the test-site / browser is set up with it:
        self.having(timezone='Etc/Greenwich')
        # Manually set start and end because an early import of IEventBasic
        # in the `test_staging.py` results to events having no start and end
        now = datetime.utcnow()
        self.starting(now)
        self.ending(now + timedelta(hours=1))

    def starting(self, date):
        if IS_PLONE_5:
            from plone.event.utils import pydt
            return self.having(start=pydt(date))
        else:
            return self.having(start=date)

    def ending(self, date):
        if IS_PLONE_5:
            from plone.event.utils import pydt
            return self.having(end=pydt(date))
        else:
            return self.having(end=date)

builder_registry.register('event page', EventPageBuilder)


class EventListingBlockBuilder(DexterityBuilder):
    portal_type = 'ftw.events.EventListingBlock'

builder_registry.register('event listing block', EventListingBlockBuilder)



from datetime import datetime
from ftw.builder import Builder
from ftw.builder import create
from ftw.events.tests import FunctionalTestCase
from ftw.simplelayout.staging.interfaces import IStaging
from ftw.testbrowser import browsing
from unittest2 import skipIf
from ftw.testing import IS_PLONE_5
from plone.event.utils import pydt
from plone.app.event.dx.behaviors import IEventBasic


@skipIf(IS_PLONE_5, 'Plone 5 has rewritten plone.app.events which makes this test obsolete')
class TestWorkingCopy(FunctionalTestCase):

    def setUp(self):
        self.portal = self.layer['portal']
        self.grant('Manager')

    @browsing
    def test_working_copy_apply_correct_event_dates(self, browser):
        start = pydt(datetime(2013, 10, 7, 9))
        end = pydt(datetime(2013, 10, 7, 16))
        new_end = pydt(datetime(2013, 10, 7, 19))

        folder = create(Builder('event folder'))
        baseline = create(Builder('event page')
                          .having(timezone='Europe/Zurich')
                          .within(folder))

        behaviour = IEventBasic(baseline)
        behaviour.start, behaviour.end = start, end

        self.assertEquals(start, behaviour.start)

        working_copy = IStaging(baseline).create_working_copy(folder)
        self.assertFalse(IStaging(baseline).is_working_copy())

        working_behaviour = IEventBasic(working_copy)
        working_behaviour.end = new_end
        IStaging(working_copy).apply_working_copy()

        self.assertEquals(new_end, behaviour.end)



from datetime import datetime
from datetime import timedelta
from ftw.builder import Builder
from ftw.builder import create
from ftw.events.tests import FunctionalTestCase
from ftw.testbrowser import browsing
import transaction


class TestRssView(FunctionalTestCase):

    def setUp(self):
        super(TestRssView, self).setUp()
        self.grant('Manager')

        self.event_folder = create(Builder('event folder')
                                   .titled(u'events'))

        self.event_folder.events.show_rss_link = True
        transaction.commit()

        # Both events have to be in the future, to achieve this we use the
        # current time and add a few days using timedelta.
        now = datetime.now()

        self.event_1 = create(Builder('event page')
                              .titled(u"An event")
                              .having(description=u'This is a test event')
                              .starting(now + timedelta(days=30))
                              .ending(now + timedelta(days=30, hours=5))
                              .within(self.event_folder))

        self.event_2 = create(Builder('event page')
                              .titled(u"A second event")
                              .having(description=u'Another test event')
                              .starting(now + timedelta(days=60))
                              .ending(now + timedelta(days=60, hours=3))
                              .within(self.event_folder))

    @browsing
    def test_rss_button_is_visible(self, browser):
        browser.login().visit(self.event_folder)
        link_to_rss = browser.css('.ftw-events-eventlistingblock .events-rss')

        self.assertEqual(['Subscribe to the RSS feed'],
                         link_to_rss.text)
        self.assertEqual('http://nohost/plone/events/events/events_rss',
                         link_to_rss.first.get('href'))

    @browsing
    def test_rss_feed_structure(self, browser):
        browser.login().visit(self.event_folder.events, view='events_rss')

        self.assertEqual('2.0',
                         browser.css('rss').first.get('version'))
        self.assertEqual(['http://nohost/plone/events/events/events_rss'],
                         browser.css('rss channel > link').text)
        self.assertEqual(['Events - Events Feed'],
                         browser.css('rss channel > description').text)
        self.assertEqual(['An event', 'A second event'],
                         browser.css('rss item title').text)
        self.assertEqual(['http://nohost/plone/events/an-event',
                          'http://nohost/plone/events/a-second-event'],
                         browser.css('rss channel item link').text)



from ftw.events import _
from ftw.events.interfaces import IEventFolder
from plone import api
from plone.autoform.interfaces import IFormFieldProvider
from plone.dexterity.content import Container
from plone.directives import form
from zope.component.hooks import getSite
from zope.i18n import translate
from zope.interface import alsoProvides
from zope.interface import implements


class IEventFolderSchema(form.Schema):
    pass

alsoProvides(IEventFolderSchema, IFormFieldProvider)


class EventFolder(Container):
    implements(IEventFolder)


def create_event_listing_block(event_folder, event=None):
    """
    This methods creates an event listing block inside the given event folder
    and is used as a handler for a subscriber listening on the creation
    of event folders.
    """
    api.content.create(
        container=event_folder,
        type='ftw.events.EventListingBlock',
        title=translate(
            _(u'title_default_eventlisting_block', u'Events'),
            context=getSite().REQUEST,
        ),
        current_context=True,
        subjects=[],
        show_title=False,
        filter_by_path=[],
    )



from ftw.builder import Builder
from ftw.builder import create
from ftw.events.tests import FunctionalTestCase
from plone.app.testing import login, TEST_USER_NAME
from zope.component import getUtility
from zope.schema.interfaces import IVocabularyFactory


def terms_for(vocabulary_name, context):
    factory = getUtility(IVocabularyFactory,
                         name=vocabulary_name)
    return dict([(term.value, term.title)
                 for term in factory(context)])


class TestVocabularies(FunctionalTestCase):

    def setUp(self):
        super(TestVocabularies, self).setUp()
        self.grant('Contributor')

    def test_subject_vocabulary(self):
        create(Builder('event page')
               .titled(u'Event 1')
               .having(subjects=['hans', u'mari\xe4']))
        create(Builder('event page')
               .titled(u'Event 2')
               .having(subjects=['peter', u'mari\xe4']))

        login(self.portal, TEST_USER_NAME)
        self.assertEquals(
            {u'hans': u'hans', u'mari\xe4': u'mari\xe4', u'peter': u'peter'},
            terms_for('ftw.events.vocabulary.subjects', self.portal))



from DateTime import DateTime
from datetime import datetime
from datetime import timedelta
from ftw.builder import Builder
from ftw.builder import create
from ftw.events.behaviors.mopage import IMopageModificationDate
from ftw.events.tests import FunctionalTestCase
from ftw.events.tests import XMLDiffTestCase
from ftw.events.tests import utils
from ftw.testbrowser import browser
from ftw.testbrowser import browsing
from ftw.testing import freeze
from ftw.testing import staticuid
from path import Path
from plone.app.textfield import RichTextValue
from pytz import timezone
import re


ZURICH = timezone('Europe/Zurich')


class TestMopageExport(FunctionalTestCase, XMLDiffTestCase):

    @staticuid('uid')
    @browsing
    def test_01_export_events(self, browser):
        self.grant('Manager')
        event_folder = create(Builder('event folder').titled(u'event folder'))

        with freeze(ZURICH.localize(datetime(2010, 3, 14, 20, 18))):
            events1 = create(
                Builder('event page')
                .titled(u'Wanderausstellung Kaffeebohnen')
                .within(event_folder)
                .starting(datetime(2010, 10, 7, 9, 00))
                .ending(datetime(2010, 10, 7, 16, 00))
                .having(recurrence='RRULE:FREQ=DAILY;COUNT=4',
                        description=u'Die Wanderausstellung Kaffeebohnen'
                        u' besch\xe4ftigt sich mit allen m\xf6glichen'
                        u' Kaffeesorten aus aller Welt.',
                        subjects=('Kaffee', 'Wanderausstellung'),
                        location_title='Kunstmuseum Bern',
                        location_street='Hodlerstrasse 8',
                        location_zip='3011',
                        location_city='Bern'))

            lorem = RichTextValue(self.asset('lorem1.html'))
            block = create(Builder('sl textblock').within(events1)
                           .with_dummy_image()
                           .having(text=lorem))
            utils.create_page_state(events1, block)
            IMopageModificationDate(events1).set_date(ZURICH.localize(datetime(2010, 3, 15)))

        with freeze(ZURICH.localize(datetime(2010, 5, 17, 15, 34))):
            create(Builder('event page')
                   .titled(u'Bratwurstgrillieren')
                   .within(event_folder)
                   .starting(datetime(2010, 5, 18))
                   .ending(datetime(2010, 5, 18))
                   .having(whole_day=True,
                           # Location is incomplete and will not appear.
                           location_title='Kunstmuseum Bern'))

        with freeze(ZURICH.localize(datetime(2011, 1, 2, 3, 4))):
            self.assert_mopage_export('mopage.events.xml', event_folder)

    @browsing
    def test_pagination(self, browser):
        self.grant('Manager')
        event_folder = create(Builder('event folder').titled(u'Events'))
        with freeze(ZURICH.localize(datetime(2015, 10, 1, 14, 0))) as clock:
            create(Builder('event page').titled(u'One').within(event_folder)
                   .starting(datetime.now() + timedelta(hours=2))
                   .ending(datetime.now() + timedelta(hours=2)))
            clock.forward(days=1)
            create(Builder('event page').titled(u'Two').within(event_folder)
                   .starting(datetime.now() + timedelta(hours=2))
                   .ending(datetime.now() + timedelta(hours=2)))
            clock.forward(days=1)
            create(Builder('event page').titled(u'Three').within(event_folder)
                   .starting(datetime.now() + timedelta(hours=2))
                   .ending(datetime.now() + timedelta(hours=2)))
            clock.forward(days=1)
            create(Builder('event page').titled(u'Four').within(event_folder)
                   .starting(datetime.now() + timedelta(hours=2))
                   .ending(datetime.now() + timedelta(hours=2)))
            clock.forward(days=1)
            create(Builder('event page').titled(u'Five').within(event_folder)
                   .starting(datetime.now() + timedelta(hours=2))
                   .ending(datetime.now() + timedelta(hours=2)))

        browser.open(event_folder, view='mopage.events.xml?per_page=2')
        self.assert_events_in_browser(['Five', 'Four'])
        links = self.get_links_from_response()
        self.assertEquals(
            {'next': 'http://nohost/plone/events/mopage.events.xml?per_page=2&page=2',
             'last': 'http://nohost/plone/events/mopage.events.xml?per_page=2&page=3'},
            links)

        browser.open(links['next'])
        self.assert_events_in_browser(['Three', 'Two'])
        links = self.get_links_from_response()
        self.assertEquals(
            {'first': 'http://nohost/plone/events/mopage.events.xml?per_page=2&page=1',
             'prev': 'http://nohost/plone/events/mopage.events.xml?per_page=2&page=1',
             'next': 'http://nohost/plone/events/mopage.events.xml?per_page=2&page=3',
             'last': 'http://nohost/plone/events/mopage.events.xml?per_page=2&page=3'},
            links)

        browser.open(links['next'])
        self.assert_events_in_browser(['One'])
        links = self.get_links_from_response()
        self.assertEquals(
            {'first': 'http://nohost/plone/events/mopage.events.xml?per_page=2&page=1',
             'prev': 'http://nohost/plone/events/mopage.events.xml?per_page=2&page=2'},
            links)

    @browsing
    def test_export_all_events_on_site(self, browser):
        self.grant('Manager')
        create(Builder('event page').titled(u'One').within(
            create(Builder('event folder').titled(u'Event Folder One'))))
        create(Builder('event page').titled(u'Two').within(
            create(Builder('event folder').titled(u'Event Folder Two'))))

        browser.open(self.portal, view='mopage.events.xml')
        self.assert_events_in_browser(['One', 'Two'])

    @browsing
    def test_title_is_cropped(self, browser):
        self.grant('Manager')
        create(Builder('event page').titled(u'A' * 150).within(
            create(Builder('event folder'))))

        browser.open(self.portal, view='mopage.events.xml')
        self.assert_events_in_browser(['A' * 95 + ' ...'])

    @browsing
    def test_include_root_arguments_when_submitted_as_GET_param(self, browser):
        self.grant('Manager')
        create(Builder('event page').within(create(Builder('event folder'))))

        date = ZURICH.localize(datetime(2016, 8, 9, 21, 45))
        with freeze(date):
            browser.open(self.portal, view='mopage.events.xml',
                         data={'partner': 'Partner',
                               'partnerid': '123',
                               'passwort': 's3c>r3t',
                               'importid': '456',
                               'vaterobjekt': 'xy',
                               'unkown_key': 'should not appear'})

        self.assertEquals(
            {
                'export_time': '2016-08-09 21:45:00',
                'partner': 'Partner',
                'partnerid': '123',
                'passwort': 's3c>r3t',
                'importid': '456',
                'vaterobjekt': 'xy',
            },
            browser.css('import').first.attrib)

    def include_root_arguments_when_submitted_as_GET_param_helper(self, browser, date):
        with freeze(date):
            browser.open(self.portal, view='mopage.events.xml',
                         data={'partner': 'Partner',
                               'partnerid': '123',
                               'passwort': 's3c>r3t',
                               'importid': '456',
                               'vaterobjekt': 'xy',
                               'unkown_key': 'should not appear'})

    def assert_mopage_export(self, asset_name, export_context):
        expected = self.asset(asset_name)
        browser.open(export_context, view='mopage.events.xml')
        got = browser.contents
        # replace dynamic scale image urls for having static test results:
        got = re.sub(r'\/@@images/[a-z0-9-]{36}.jpeg', '/image.jpg', got)
        # remove trailing spaces:
        got = re.sub(r' +$', '', got, flags=re.M)
        self.maxDiff = None
        self.assert_xml(expected, got)

    def asset(self, asset_name):
        assets = Path(__file__).joinpath('..', 'mopage_assets').abspath()
        return assets.joinpath(asset_name).bytes()

    def assert_events_in_browser(self, expected_titles):
        got_titles = browser.css('titel').text
        self.assertEquals(expected_titles, got_titles)

    def get_links_from_response(self):
        def parse_link(text):
            return tuple(reversed(re.match(
                r'^<([^>]+)>; rel="([^"]+)"', text).groups()))

        return dict(map(parse_link, browser.headers.get('Link').split(',')))



from ftw.upgrade import UpgradeStep


class AddObjectActionForExportingICSFile(UpgradeStep):
    """Add object action for exporting ICS file.
    """

    def __call__(self):
        self.install_upgrade_profile()



from ftw.events.contents.eventpage import IEventPageSchema
from ftw.events.interfaces import IEventPage
from plone.app.event.dx.behaviors import EventAccessor
from plone.event.interfaces import IEventAccessor
from zope.component import adapter
from zope.interface import implementer


@adapter(IEventPageSchema)
@implementer(IEventAccessor)
class FtwEventAccessor(EventAccessor):
    def __init__(self, context):
        super(FtwEventAccessor, self).__init__(context)
        self._behavior_map['location'] = IEventPage



from plone.app.layout.viewlets import ViewletBase
from plone.event.interfaces import IRecurrenceSupport
from zope.component import getMultiAdapter
from zope.contentprovider.interfaces import IContentProvider


class EventDetailsViewlet(ViewletBase):

    def events(self):
        return IRecurrenceSupport(self.context).occurrences()

    def formatted_date(self, occurance):
        provider = getMultiAdapter((self.context, self.request, self.view),
                                   IContentProvider, name=u'formatted_date')
        return provider(occurance)



from ftw.builder import Builder
from ftw.builder import create
from ftw.events.contents.eventlistingblock import IEventListingBlockSchema
from ftw.events.tests import FunctionalTestCase
from ftw.events.tests.utils import enable_behavior
from ftw.testbrowser import browsing
from ftw.testbrowser.pages import factoriesmenu
from ftw.testbrowser.pages import statusmessages
from ftw.testing import freeze
from plone import api
from plone.protect.authenticator import createToken
from z3c.relationfield import RelationValue
from zope.component import getUtility
from zope.intid import IIntIds
from Products.CMFPlone.interfaces.syndication import IFeedSettings
import datetime
import transaction


class TestEventListingBlock(FunctionalTestCase):

    def setUp(self):
        super(TestEventListingBlock, self).setUp()
        self.grant('Manager')

    def _get_event_titles_from_block(self, browser):
        """
        This helper method returns the titles of the events rendered
        in the event listing block. Use this method in the test case to
        make sure the block lists only the events we want it to.
        """
        return browser.css('.ftw-events-eventlistingblock .event-row h3.title').text

    @browsing
    def test_event_listing_block_builder(self, browser):
        page = create(Builder('sl content page').titled(u'A page'))

        create(Builder('event listing block')
               .within(page)
               .titled(u'This block renders event pages'))

        browser.login().visit(page)
        self.assertEqual(
            ['This block renders event pages'],
            browser.css('.sl-block.ftw-events-eventlistingblock h2').text
        )

    @browsing
    def test_event_listing_block_can_be_added_on_contentpage(self, browser):
        page = create(Builder('sl content page').titled(u'A page'))
        browser.login().visit(page)
        browser.append_request_header('X-CSRF-TOKEN', createToken())

        factoriesmenu.add('EventListingBlock')
        browser.fill({
            'Title': u'This block renders event pages',
        }).save()

        browser.open(page)
        self.assertEqual(
            ['This block renders event pages'],
            browser.css('.sl-block.ftw-events-eventlistingblock h2').text
        )

    @browsing
    def test_event_listing_block_does_not_render_title(self, browser):
        page = create(Builder('sl content page').titled(u'A page'))

        block = create(Builder('event listing block')
                       .within(page)
                       .titled(u'This block renders event pages'))
        browser.login()
        browser.append_request_header('X-CSRF-TOKEN', createToken())

        # Make sure the title is rendered by default.
        browser.visit(page)
        self.assertEqual(
            ['This block renders event pages'],
            browser.css('.sl-block.ftw-events-eventlistingblock h2').text
        )

        # Tell the block to no longer display the title.
        browser.login().visit(block, view='edit.json')
        response = browser.json
        browser.parse(response['content'])
        browser.fill({
            'Show title': False,
        })
        browser.find_button_by_label('Save').click()

        # Make sure the title is no longer rendered.
        browser.visit(page)
        self.assertEqual(
            [],
            browser.css('.sl-block.ftw-events-eventlistingblock h2')
        )

    @browsing
    def test_sort_order(self, browser):
        """
        This test makes sure that the events are sorted in ascending order by
        their start date.
        """
        event_folder = create(Builder('event folder').titled(u'Event Folder'))
        with freeze(datetime.datetime(2010, 7, 1)):
            create(Builder('event page')
                   .titled(u'Event of Hans')
                   .starting(datetime.datetime(2010, 7, 2))
                   .ending(datetime.datetime(2010, 7, 3))
                   .within(event_folder))
            create(Builder('event page')
                   .titled(u'Event of Peter')
                   .starting(datetime.datetime(2010, 7, 1))
                   .ending(datetime.datetime(2010, 7, 5))
                   .within(event_folder))

            browser.login()

            # Make sure that the events are sorted in ascending order.
            browser.visit(event_folder)
            self.assertEqual(
                ['Event of Peter', 'Event of Hans'],
                self._get_event_titles_from_block(browser),
                "Event of Peter must be listed before the event of Hans"
            )

    @browsing
    def test_block_filters_by_path(self, browser):
        """
        This test makes sure that the block only renders items
        from the given path.
        """
        event_folder1 = create(Builder('event folder')
                               .titled(u'Event Folder 1')
                               .within(self.portal))
        create(Builder('event page').titled(u'Hello World 1').within(event_folder1))

        event_folder2 = create(Builder('event folder')
                               .titled(u'Event Folder 2')
                               .within(self.portal))
        create(Builder('event page').titled(u'Hello World 2').within(event_folder2))

        # Prepare a page containing an event listing block.
        page = create(Builder('sl content page').titled(u'Content Page with Eventlistingblock'))
        block = create(Builder('event listing block')
                       .within(page)
                       .having(current_context=False)
                       .titled(u'Not relevant in this test'))

        browser.login()
        browser.append_request_header('X-CSRF-TOKEN', createToken())

        # Edit the block and set a path (not possible with builder).
        browser.visit(block, view='edit.json')
        response = browser.json
        browser.parse(response['content'])
        browser.fill({
            'Limit to path': event_folder2,
        })
        browser.find_button_by_label('Save').click()
        browser.visit(page)

        # Make sure the block only lists events from the path
        # we have configured on the block.
        self.assertEqual(
            ['Hello World 2'],
            self._get_event_titles_from_block(browser)
        )

    @browsing
    def test_block_prevents_path_and_current_context(self, browser):
        """
        This test makes sure that a block cannot be configured to
        have a "filter by path" and a "filter by current context" at the
        same time.
        """
        event_folder = create(Builder('event folder').titled(u'Event Folder'))

        browser.login()

        # Note that a default event listing block is created
        # automatically in the event folder. Let's get the block.
        block = event_folder.listFolderContents(
            contentFilter={'portal_type': 'ftw.events.EventListingBlock'}
        )[0]

        # Edit the existing block.
        browser.visit(block, view='edit.json')
        response = browser.json
        browser.parse(response['content'])
        browser.fill({
            'Title': u'Not relevant for this test',
            'Limit to path': '/'.join(event_folder.getPhysicalPath()),
            'Limit to current context': True,
        })
        browser.find_button_by_label('Save').click()
        response = browser.json
        browser.parse(response['content'])

        # Make sure that the form refuses to save.
        self.assertEqual(
            ['There were some errors.'],
            statusmessages.error_messages()
        )
        self.assertIn(
            'You cannot filter by path and current context at the same time.',
            browser.css('.error').text
        )

    @browsing
    def test_block_filter_by_current_context(self, browser):
        """
        This test makes sure the block filters by the current context.
        """
        page1 = create(Builder('sl content page').titled(u'Content Page 1'))
        event_folder1 = create(Builder('event folder')
                               .titled(u'Event Folder 1')
                               .within(page1))
        create(Builder('event page').titled(u'Hello World 1').within(event_folder1))

        page2 = create(Builder('sl content page').titled(u'Content Page 2'))
        event_folder2 = create(Builder('event folder')
                               .titled(u'Event Folder 2')
                               .within(page2))
        create(Builder('event page').titled(u'Hello World 2').within(event_folder2))

        block = create(Builder('event listing block')
                       .within(page2)
                       .titled(u'Not relevant for this test'))

        browser.login()
        browser.append_request_header('X-CSRF-TOKEN', createToken())

        # Make sure that the filter by context is active by default.
        browser.visit(page2)
        self.assertEqual(
            ['Hello World 2'],
            self._get_event_titles_from_block(browser),
        )

        # Tell the block to no longer filter by the current context.
        browser.login().visit(block, view='edit.json')
        response = browser.json
        browser.parse(response['content'])
        browser.fill({
            'Limit to current context': False,
        })
        browser.find_button_by_label('Save').click()

        # Make sure the title is no longer rendered.
        browser.visit(page2)
        self.assertEqual(
            ['Hello World 1', 'Hello World 2'],
            self._get_event_titles_from_block(browser),
        )

    @browsing
    def test_block_crops_description(self, browser):
        """
        This test makes sure that the block crops the description of the event
        page to the default length (note that the description is shown by default
        but can be disabled).
        """
        page = create(Builder('sl content page').titled(u'Content Page 1'))
        event_folder = create(Builder('event folder').titled(u'Event Folder 1').within(page))
        create(Builder('event page')
               .titled(u'Hello World 1')
               .having(description=u"This description must be longer than "
                                   u"50 characters so we are able to test "
                                   u"if it will be cropped.")
               .within(event_folder))
        create(Builder('event listing block')
               .within(page)
               .titled(u'This is a EventListingBlock'))

        browser.login().visit(page)
        self.assertEqual(
            ['This description must be longer than 50 ...'],
            browser.css('.sl-block.ftw-events-eventlistingblock .description').text
        )

    @browsing
    def test_block_renders_entire_description(self, browser):
        """
        This test makes sure that the block renders the entire description
        if configured to do so.
        """
        page = create(Builder('sl content page').titled(u'Content Page 1'))
        event_folder = create(Builder('event folder').titled(u'Event Folder 1').within(page))
        create(Builder('event page')
               .titled(u'Hello World 1')
               .having(description=u"This description must be longer than "
                                   u"50 characters so we are able to test "
                                   u"if it will be cropped.")
               .within(event_folder))
        create(Builder('event listing block')
               .within(page)
               .having(description_length=0)
               .titled(u'This is a EventListingBlock'))

        browser.login().visit(page)
        self.assertEqual(
            [
                'This description must be longer than 50 characters '
                'so we are able to test if it will be cropped.'
            ],
            browser.css('.sl-block.ftw-events-eventlistingblock .description').text
        )

    @browsing
    def test_block_does_not_render_description(self, browser):
        """
        This test makes sure that the block does not render the description
        if configured to do so.
        """
        page = create(Builder('sl content page').titled(u'Content Page'))
        event_folder = create(Builder('event folder').titled(u'Event Folder').within(page))
        create(Builder('event page')
               .titled(u'Hello World')
               .having(description=u"The description.")
               .within(event_folder))
        create(Builder('event listing block')
               .within(page)
               .having(show_description=False)
               .titled(u'This is a EventListingBlock'))

        browser.login().visit(page)
        self.assertEqual(
            [],
            browser.css('.sl-block.ftw-events-eventlistingblock .description').text
        )

    @browsing
    def test_block_filters_by_subject(self, browser):
        """
        This test makes sure that the block filters event pages by subject.
        """
        page = create(Builder('sl content page').titled(u'Content Page'))
        event_folder = create(Builder('event folder').titled(u'Event Folder').within(page))
        create(Builder('event listing block')
               .within(page)
               .having(subjects=[u'H\xe4ns'])
               .titled(u'This is a EventListingBlock'))
        with freeze(datetime.datetime(2010, 7, 1)):
            create(Builder('event page')
                   .titled(u'Event of Hans')
                   .having(subjects=[u'H\xe4ns'])
                   .starting(datetime.datetime(2010, 7, 2))
                   .ending(datetime.datetime(2010, 7, 2))
                   .within(event_folder))
            create(Builder('event page')
                   .titled(u'Event of Peter')
                   .having(subjects=['Peter'])
                   .starting(datetime.datetime(2010, 7, 2))
                   .ending(datetime.datetime(2010, 7, 2))
                   .within(event_folder))

            browser.login()
            browser.visit(page)
            self.assertEqual(
                ['Event of Hans'],
                self._get_event_titles_from_block(browser)
            )

    @browsing
    def test_block_limits_quantity(self, browser):
        page = create(Builder('sl content page').titled(u'Content Page'))
        event_folder = create(Builder('event folder').titled(u'Event Folder').within(page))
        quantity = 1
        create(Builder('event listing block')
               .within(page)
               .having(quantity=quantity)
               .having(show_title=False)
               .titled(u'This is a EventListingBlock'))

        with freeze(datetime.datetime(2010, 7, 1)):
            create(Builder('event page')
                   .titled(u'Event of Hans')
                   .starting(datetime.datetime(2010, 7, 2))
                   .ending(datetime.datetime(2010, 7, 2))
                   .within(event_folder))
            create(Builder('event page')
                   .titled(u'Event of Peter')
                   .starting(datetime.datetime(2010, 7, 2))
                   .ending(datetime.datetime(2010, 7, 2))
                   .within(event_folder))

            browser.login()
            browser.visit(page)
            self.assertEqual(
                quantity,
                len(self._get_event_titles_from_block(browser))
            )

    @browsing
    def test_block_renders_link_to_more_items(self, browser):
        page = create(Builder('sl content page')
                      .titled(u'Content Page'))
        event_folder = create(Builder('event folder').
                              titled(u'Event Folder')
                              .within(page))
        create(Builder('event page')
               .titled(u'Event')
               .within(event_folder))
        block = create(Builder('event listing block')
                       .within(page)
                       .having(show_more_items_link=True)
                       .titled(u'This is a EventListingBlock'))

        browser.login()
        browser.append_request_header('X-CSRF-TOKEN', createToken())

        # Make sure the link is there.
        browser.visit(page)
        self.assertEqual(
            'http://nohost/plone/content-page/this-is-a-eventlistingblock/events',
            browser.find('More Items').attrib['href']
        )

        # Tell the block to no longer render the link
        browser.login().visit(block, view='edit.json')
        response = browser.json
        browser.parse(response['content'])
        browser.fill({
            'Show link to more items': False,
        })
        browser.find_button_by_label('Save').click()

        # Make sure the link is no longer there.
        browser.visit(page)
        self.assertIsNone(browser.find('More Items'))

    @browsing
    def test_block_renders_link_to_more_items_with_custom_label(self, browser):
        """
        This test makes sure that the link to thew view which renders more items
        has a custom label.
        """
        page = create(Builder('sl content page')
                      .titled(u'Content Page'))
        event_folder = create(Builder('event folder').
                              titled(u'Event Folder')
                              .within(page))
        create(Builder('event page')
               .titled(u'Event')
               .within(event_folder))
        create(Builder('event listing block')
               .within(page)
               .having(show_more_items_link=True)
               .having(more_items_link_label=u'Show me more')
               .titled(u'This is a EventListingBlock'))

        browser.login()

        # Make sure the link is there.
        browser.visit(page)
        self.assertEqual(
            'http://nohost/plone/content-page/this-is-a-eventlistingblock/events',
            browser.find('Show me more').attrib['href']
        )

    @browsing
    def test_custom_link_to_show_more_items(self, browser):

        page = create(Builder('sl content page')
                      .titled(u'Content Page'))

        page_to_link = create(Builder('sl content page')
                              .titled(u'Content Page'))

        event_folder = create(Builder('event folder').
                              titled(u'Event Folder')
                              .within(page))
        create(Builder('event page')
               .titled(u'Event')
               .within(event_folder))
        block = create(Builder('event listing block')
                       .within(page)
                       .having(show_more_items_link=True)
                       .titled(u'This is a EventListingBlock'))

        browser.login().visit(page)
        self.assertEqual(
            'http://nohost/plone/content-page/this-is-a-eventlistingblock/events',
            browser.find('More Items').attrib['href']
        )

        browser.login().visit(block, view='@@edit')
        browser.fill({'Link to more items': page_to_link}).submit()

        self.assertEqual(
            page_to_link.absolute_url(),
            browser.find('More Items').attrib['href']
        )

    @browsing
    def test_custom_link_to_show_more_items_when_target_is_deleted(self, browser):

        # Create an event folder within the Plone Site.
        event_folder = create(Builder('event folder').
                              titled(u'Event Folder')
                              .within(self.portal))

        # Create an event page within the event folder so the event listing block
        # has something to render. Empty event listing blocks will never render the
        # "link to more items.
        create(Builder('event page')
               .within(event_folder))

        # Create a content page we will use as the "link to more items".
        target = create(Builder('sl content page')
                        .titled(u'Target of the link to more items'))

        # Get the event folder's event listing block which has been created
        # automatically.
        event_listing_block = api.content.find(
            portal_type='ftw.events.EventListingBlock',
            within=event_folder
        )[0].getObject()

        # Customize the "link to more items" of the event listing block so it points
        # to the content page we just created and tell the block to render the link.
        IEventListingBlockSchema(event_listing_block).link_to_more_items = RelationValue(
            getUtility(IIntIds).getId(target)
        )
        IEventListingBlockSchema(event_listing_block).show_more_items_link = True
        transaction.commit()

        # The link is rendered on the event list block and links to our target content page.
        # This is the expected behavior before we are going to deleted the content page.
        browser.login().visit(event_folder)
        self.assertEqual(
            'http://nohost/plone/target-of-the-link-to-more-items',
            browser.find('More Items').attrib['href']
        )

        # Somebody deletes the content page we used as the target for the "link to more items".
        self.portal.manage_delObjects(ids=[target.getId()])
        transaction.commit()

        # Open the page again.
        browser.login().visit(event_folder)

        # Now the link points to events view on the event listing block.
        self.assertEqual(
            'http://nohost/plone/event-folder/events/events',
            browser.find('More Items').attrib['href']
        )

    def test_syndication_is_enabled_by_default_on_block(self):
        folder = create(Builder('event folder'))

        # Note that a default event listing block is created
        # automatically in the event folder.
        block = folder.listFolderContents(
            contentFilter={'portal_type': 'ftw.events.EventListingBlock'}
        )[0]

        self.assertTrue(
            IFeedSettings(block).enabled
        )
        self.assertEqual(
            ('RSS', 'rss.xml', 'atom.xml'),
            IFeedSettings(block).feed_types
        )

    @browsing
    def test_block_renders_location(self, browser):
        event_folder = create(Builder('event folder'))
        event = create(Builder('event page')
                       .titled(u'My Event')
                       .having(location_title='Infinite Loop 1')
                       .within(event_folder))
        browser.login()

        # Make sure the location is rendered.
        browser.open(event_folder)
        self.assertEqual(
            ['Infinite Loop 1'],
            browser.css('.event-row .byline .location').text
        )

        # Empty the location and make sure it is no longer rendered.
        browser.visit(event, view='edit')
        browser.fill({'Location: title': u''}).submit()
        browser.open(event_folder)
        self.assertEqual(
            [],
            browser.css('.event-row .byline .location')
        )

    @browsing
    def test_block_does_not_render_past_events(self, browser):
        page = create(Builder('sl content page'))
        event_folder = create(Builder('event folder').titled(u'Event Folder').within(page))
        create(Builder('event listing block')
               .within(page)
               .titled(u'Event listing block'))

        with freeze(datetime.datetime(2010, 7, 1, 15, 0, 0)):
            create(Builder('event page')
                   .titled(u'Past event')
                   .starting(datetime.datetime(2010, 1, 1))
                   .ending(datetime.datetime(2010, 1, 1))
                   .within(event_folder))
            create(Builder('event page')
                   .titled(u'Today event')
                   .starting(datetime.datetime(2010, 7, 1, 12, 0, 0))
                   .ending(datetime.datetime(2010, 7, 1, 18, 0, 0))
                   .within(event_folder))
            create(Builder('event page')
                   .titled(u'Future event')
                   .starting(datetime.datetime(2010, 7, 2))
                   .ending(datetime.datetime(2010, 7, 2))
                   .within(event_folder))
            create(Builder('event page')
                   .titled(u'Event which ended a few hours ago')
                   .starting(datetime.datetime(2010, 7, 1, 9, 0, 0))
                   .ending(datetime.datetime(2010, 7, 1, 14, 0, 0))
                   .within(event_folder))

            browser.login()
            browser.visit(page)
            self.assertEqual(
                ['Event which ended a few hours ago', 'Today event', 'Future event'],
                self._get_event_titles_from_block(browser)
            )

    @browsing
    def test_contributor_can_see_inactive_events_in_event_listing_block(self, browser):
        enable_behavior('plone.app.dexterity.behaviors.metadata.IPublication', 'ftw.events.EventPage')

        page = create(Builder('sl content page').titled(u'Content Page'))
        create(Builder('event listing block')
               .within(page)
               .titled(u'Not relevant for this test'))

        event_folder = create(Builder('event folder').within(page))
        create(Builder('event page')
               .titled(u'Future Event')
               .within(event_folder)
               .having(effective=datetime.datetime.now() + datetime.timedelta(days=10)))

        # Make sure a contributor can see inactive news.
        contributor = create(Builder('user').named('A', 'Contributor').with_roles('Contributor'))
        browser.login(contributor).visit(page)
        self.assertEqual(
            ['Future Event'],
            self._get_event_titles_from_block(browser),
        )

        # Make sure an anonymous user does not see the inactive news.
        browser.logout().visit(page)
        self.assertEquals(
            [],
            self._get_event_titles_from_block(browser),
        )

    @browsing
    def test_block_without_events_can_be_marked_as_hidden(self, browser):
        """
        This test makes sure that there is a CSS class "hidden" on the block
        if the block is empty and the block has been configured accordingly.
        """
        page = create(Builder('sl content page'))
        block = create(Builder('event listing block')
                       .within(page)
                       .titled(u'Event listing block'))

        def _block_has_hidden_class(browser):
            return 'hidden' in browser.css('.ftw-events-eventlistingblock').first.attrib['class']

        browser.login()
        browser.append_request_header('X-CSRF-TOKEN', createToken())

        # Make sure the block has no "hidden" class.
        browser.visit(page)
        self.assertFalse(_block_has_hidden_class(browser))

        # Edit the block.
        browser.visit(block, view='edit')
        browser.fill({u'Hide empty block': True}).find('Save').click()

        # Make sure the block has a "hidden" class now.
        browser.visit(page)
        self.assertTrue(_block_has_hidden_class(browser))



from Products.CMFCore.utils import getToolByName
from ftw.simplelayout.interfaces import IPageConfiguration
from ftw.testing import IS_PLONE_5
from plone import api
from plone.registry.interfaces import IRegistry
from plone.uuid.interfaces import IUUID
from zope.component import getUtility
from zope.component.hooks import getSite
import transaction


def create_page_state(obj, block):
    page_state = {
        "default": [
            {
                "cols": [
                    {
                        "blocks": [
                            {
                                "uid": IUUID(block)
                            }
                        ]
                    }
                ]
            },
        ]
    }
    page_config = IPageConfiguration(obj)
    page_config.store(page_state)
    transaction.commit()


def set_allow_anonymous_view_about(state):
    site_props = api.portal.get_tool(name='portal_properties').site_properties
    site_props.allowAnonymousViewAbout = state
    transaction.commit()


def enable_behavior(behavior, portal_type):
    portal_types = getToolByName(getSite(), 'portal_types')
    behaviors = list(portal_types[portal_type].behaviors)
    behaviors.append(behavior)
    portal_types[portal_type].behaviors = tuple(behaviors)
    transaction.commit()


class LanguageSetter(object):

    def set_language_settings(self, default='en', supported=None,
                              use_combined=False, start_neutral=True):
        """
        Sets language settings regardeless if plone4.3 or plone5.1
        :param default: default site language
        :param supported: list of supported languages
        """
        # startNeutral is not used/available in plone 5.1 anymore

        if not supported:
            supported = ['en']

        if IS_PLONE_5:
            from Products.CMFPlone.interfaces import ILanguageSchema

            self.ltool = api.portal.get_tool('portal_languages')
            self.ltool.setDefaultLanguage(default)
            for lang in supported:
                self.ltool.addSupportedLanguage(lang)
            self.ltool.settings.use_combined_language_codes = False
            self.ltool.setLanguageCookie()
            registry = getUtility(IRegistry)
            language_settings = registry.forInterface(ILanguageSchema, prefix='plone')
            language_settings.use_content_negotiation = True
        else:
            self.ltool = self.portal.portal_languages
            self.ltool.manage_setLanguageSettings(
                default,
                supported,
                setUseCombinedLanguageCodes=use_combined,
                # Set this only for better testing ability
                setCookieEverywhere=True,
                startNeutral=start_neutral,
                setContentN=True)
        transaction.commit()
